from flask import render_template, abort, redirect, url_for, abort, flash, request, current_app, make_response, g
from flask_login import login_required, login_user, logout_user
from flask_sqlalchemy import get_debug_queries

from . import admin
from .forms import ChargeForm

from .. import db
from ..email import send_email
from ..models import User, Deployment, Service, Region, Address, Domain, contact_proxmaster
from ..decorators import admin_required, permission_required

import base64
import string
import random
from datetime import datetime, timedelta, date, time
import ast

#@admin.before_app_request
#def before_request():
#    g.user = current_user
#    print('current_user: %s, g.user: %s, leaving bef_req' % (current_user, g.user))

@admin.after_app_request
def after_request(response):
    for query in get_debug_queries():
        if query.duration >= current_app.config['SLOW_DB_QUERY_TIME']:
            current_app.logger.warning('Slow query: %s\nParameters: %s\nDuration: %fs\nContext: %s\n' % (query.statement, query.parameters, query.duration, query.context))
    return response

@admin.route("/", methods=['GET'])
@login_required
@admin_required
def index():
    alldeployments = Deployment.query.order_by(Deployment.user_id.desc()).all()
    alldomains = Domain.query.order_by(Domain.user_id.desc()).all()
    allservices = Service.query.order_by(Service.user_id.desc()).all()
    alladdresses = Address.query.order_by(Address.user_id.asc()).all()
    return render_template('admin/index.html', deployments=alldeployments, domains=alldomains, services=allservices, addresses=alladdresses)

@admin.route("/users", methods=['GET'])
@login_required
@admin_required
def users():
    allusers = User.query.order_by(User.pid.asc()).all()
    return render_template('admin/users.html', users=allusers)

@admin.route("/charge/<int:user_pid>", methods=['GET', 'POST'])
@login_required
@admin_required
def charge(user_pid=0):
    cuser = User.query.filter_by(pid=user_pid).first()
    form = ChargeForm()
    if form.validate_on_submit():
        cuser.wallet += float(form.amount.data)
        db.session.add(cuser)
        db.session.commit()
        return redirect(url_for('admin.users'))
    return render_template('admin/charge.html', form=form, usr=cuser)

@admin.route("/dashboard/<int:user_pid>", methods=['GET', 'POST'])
@login_required
@admin_required
def dashboard(user_pid=0):
    cuser = User.query.filter_by(pid=user_pid).first()

    deployments = cuser.inv_deployments.order_by(Deployment.date_created.desc()).all()
    inv_deploycubeids = []
    inv_deploynames = []
    for invcls in deployments:
        if invcls.user_id == cuser.pid and invcls.enabled == True:
            inv_deploycubeids.extend([invcls.machine_id])
            inv_deploynames.extend([invcls.machine_alias])

    services = cuser.inv_services.order_by(Service.date_created.desc()).all()
    inv_services = []
    for invcls in services:
        if invcls.user_id == cuser.pid and invcls.enabled == True:
            inv_services.extend([invcls.description])
    
    domains = cuser.inv_domains.order_by(Domain.date_created.desc()).all()
    inv_domains = []
    for invcls in domains:
        if invcls.user_id == cuser.pid and invcls.enabled == True:
            inv_domains.extend([invcls.fqdn])

    addresses = cuser.inv_addresses.order_by(Address.ip.asc()).all()
    inv_addresses = []
    for invcls in addresses:
        if invcls.user_id == cuser.pid and invcls.enabled == True:
            inv_addresses.extend([invcls.ip])

    #extract rrd and status from the deployments
    rrd = {}
    statuses = {}
    for cubeid in inv_deploycubeids:
        rrd[cubeid] = {}
        try:
            query = contact_proxmaster({}, 'vmrrd', cubeid)
        except:
            flash('Deploy #{} unreachable.'.format(str(cubeid)))

        graphs_list = ['net', 'cpu', 'mem', 'hdd']
        try: 
            for graph in graphs_list:
                raw = query[graph]['image'].encode('raw_unicode_escape')
                rrd[cubeid][graph] = base64.b64encode(raw).decode()
            status = { cubeid : query['status'] }
            statuses.update(status)
        except Exception as e:
            print(e)
            flash('Deploy #{} unreachable. Support is notified'.format(str(cubeid)))
            send_email(current_app.config['MAIL_USERNAME'], 'Cube {} is unreachable'.format(cubeid), 
                                          'vmanager/email/adm_unreachable', user=cuser, cubeid=cubeid )

    current_app.logger.info('[{}] deployments: {}, domains: {}, services: {}'.format(cuser.email, inv_deploynames, inv_services, inv_domains, inv_addresses ))
    return render_template('vmanager/dashboard.html', rrd=rrd, status=statuses, inv_deployments=deployments, inv_services=services, inv_domains=domains, inv_addresses=addresses)


